/*
 * AD840x.h
 *
 *  Created on: 25.11.2010
 *      Author: Frank Link
 */
#include <avr/io.h>
#include <util/delay.h>
#include <inttypes.h>

#ifndef _AD840x_H_
#define _AD840x_H_

//#define AD8400
#define AD8402
//#define AD8404

/*
 	Pinbelegung AD8400
	CS		PIN  3
	SDI		PIN  4
	CLK		PIN  5

 	Pinbelegung AD8402
	SHDN	PIN  6
	CS		PIN  7
	SDI		PIN  8
	CLK		PIN  9
	RS		PIN 10

 	Pinbelegung AD8404
	SHDN	PIN 10
	CS		PIN 11
	SDI		PIN 12
	SDO		PIN 13	wird derzeit noch nicht untersttzt
	CLK		PIN 14
	RS		PIN 15
*/

// Anpassung der Pinbelegung
#define AD840x_DDR	DDRC
#define AD840x_PORT	PORTC
#define AD840x_CS	PINC1
#define AD840x_SDI	PINC4
#define AD840x_CLK	PINC3

#ifndef AD8400
#define AD840x_SHDN	PINC0
#define AD840x_RS	PINC2
#endif

//------- core ------------ core ----------- core ------------- core ----------- core --------------

#if ( defined(AD8400) && defined(AD8402) ) || \
	( defined(AD8400) && defined(AD8404) ) || \
	( defined(AD8402) && defined(AD8404) ) || ( defined(AD8400) && defined(AD8402) && defined(AD8404) )
#error "Es darf nur einer von 3 defines gesetzt sein.... "
#endif

#ifndef AD8400
#define AD840x_SET_OUT   ( ( 1 << AD840x_SHDN ) | \
		                   ( 1 << AD840x_CS ) | \
	                       ( 1 << AD840x_RS ) | \
	                       ( 1 << AD840x_CLK ) | \
	                       ( 1 << AD840x_SDI ) )
#endif
#ifdef AD8400
#define AD840x_SET_OUT   ( ( 1 << AD840x_CS ) | \
	                       ( 1 << AD840x_CLK ) | \
	                       ( 1 << AD840x_SDI ) )
#endif

#define AD840x_CLK_STROBE	AD840x_PORT |= ( 1 << AD840x_CLK ); \
	                        _delay_us( 2 ); \
	                        AD840x_PORT &= ~( 1 << AD840x_CLK )

#ifndef AD8400
#define AD840x_RS_STROBE	AD840x_PORT &= ~( 1 << AD840x_RS ); \
	                        _delay_us( 2 ); \
	                        AD840x_PORT |= ( 1 << AD840x_RS )
#endif

#define AD840x_CS_ON		AD840x_PORT |= ( 1 << AD840x_CS )
#define AD840x_SDI_ON		AD840x_PORT |= ( 1 << AD840x_SDI )
#ifndef AD8400
#define AD840x_SHDN_ON		AD840x_PORT |= ( 1 << AD840x_SHDN )
#define AD840x_RS_ON		AD840x_PORT |= ( 1 << AD840x_RS )
#endif

#define AD840x_CS_OFF		AD840x_PORT &= ~( 1 << AD840x_CS )
#define AD840x_SDI_OFF		AD840x_PORT &= ~( 1 << AD840x_SDI )
#ifndef AD8400
#define AD840x_RS_OFF		AD840x_PORT &= ~( 1 << AD840x_RS )
#define AD840x_SHDN_OFF		AD840x_PORT &= ~( 1 << AD840x_SHDN )
#endif

#define AD840x_MUTE_ON				1
#define AD840x_MUTE_OFF				0

#ifdef AD8400
#define AD840x_NUMBERS				1
#endif

#ifdef AD8402
#define AD840x_NUMBERS				2
#endif

#ifdef AD8404
#define AD840x_NUMBERS				4
#endif

#endif /* AD840x_H_ */

#ifdef AD8400
extern uint8_t currentVolume;
#endif

void initAD840x(void);
void setAD840xVolume( uint8_t volume );
#ifdef AD8404
void setAD840xVolumeSet1( uint8_t volume );
void setAD840xVolumeSet2( uint8_t volume );
#endif
void setAD840xMute( uint8_t on );
